DROP TABLE IF EXISTS Person CASCADE;
DROP TABLE IF EXISTS Posting CASCADE;
DROP TABLE IF EXISTS Information CASCADE;
DROP TABLE IF EXISTS Data CASCADE;
DROP TABLE IF EXISTS Text CASCADE;
DROP TABLE IF EXISTS Follows CASCADE;
DROP TABLE IF EXISTS Tags CASCADE;
DROP TABLE IF EXISTS Likes CASCADE;

CREATE TABLE Person (
    Nickname      TEXT,
    Firstname     TEXT,
    Lastname      TEXT,
    Password      TEXT NOT NULL,
    Birthdate     TIMESTAMP,
    CONSTRAINT person_pk PRIMARY KEY (Nickname),
    CONSTRAINT password_length_check CHECK (char_length(Password)>0)
);

CREATE TABLE Posting (
    Nickname  TEXT NOT NULL,
    PostingID BIGINT NOT NULL,
    Time      TIMESTAMP NOT NULL,
    location  TEXT,
    CONSTRAINT posting_person_fk FOREIGN KEY(Nickname)
        REFERENCES Person(Nickname),
    CONSTRAINT posting_pk PRIMARY KEY(Nickname, PostingID)
);

CREATE TABLE Information (
    InformationID BIGINT,
    Nickname      TEXT    NOT NULL,
    PostingID     BIGINT  NOT NULL,
    size          BIGINT,
    CONSTRAINT information_posting_fk FOREIGN KEY(Nickname, PostingID)
        REFERENCES Posting(Nickname, PostingID),
    CONSTRAINT information_pk PRIMARY KEY(InformationID)
);

CREATE TABLE Data (
    InformationID BIGINT  NOT NULL,
    Link          TEXT    NOT NULL,
    CONSTRAINT photo_information_fk FOREIGN KEY(InformationID)
        REFERENCES Information(InformationID),
    CONSTRAINT photo_pk PRIMARY KEY(InformationID),
    CONSTRAINT link_length_check CHECK (char_length(Link)>0)
);

CREATE TABLE Text (
    InformationID BIGINT,
    Message       TEXT    NOT NULL,
    CONSTRAINT text_information_fk FOREIGN KEY(InformationID)
        REFERENCES Information(InformationID),
    CONSTRAINT text_pk PRIMARY KEY(InformationID),
    CONSTRAINT message_length_check CHECK (char_length(Message)>0)
);

CREATE TABLE Follows (
    FollowerNickname TEXT NOT NULL,
    FolloweeNickname TEXT NOT NULL,
    CONSTRAINT follows_person_follower_fk FOREIGN KEY(FollowerNickname)
        REFERENCES Person(Nickname),
    CONSTRAINT follows_person_followee_fk FOREIGN KEY(FolloweeNickname)
        REFERENCES Person(Nickname),
    CONSTRAINT follows_pk PRIMARY KEY(FollowerNickname, FolloweeNickname)
);

CREATE TABLE Likes (
    PostNickname TEXT,
    PostPostingID BIGINT,
    LikeeNickname TEXT,
    FOREIGN KEY(PostNickname, PostPostingID)
        REFERENCES Posting(Nickname, PostingID),
    FOREIGN KEY(LikeeNickname)
        REFERENCES Person(Nickname),
    PRIMARY KEY(PostNickname, PostPostingID, LikeeNickname)
);

CREATE TABLE Tags (
    Nickname  TEXT    NOT NULL,
    PostingID BIGINT  NOT NULL,
    Tag       TEXT    NOT NULL,
    CONSTRAINT tag_posting_fk FOREIGN KEY(Nickname, PostingID)
        REFERENCES Posting(Nickname, PostingID),
    CONSTRAINT tag_pk PRIMARY KEY(Nickname, PostingID, Tag),
    CONSTRAINT tag_length_check CHECK (char_length(Tag)>0)
);
